/******************************************************************************
 * Copyright 1991 Advanced Micro Devices, Inc.
 *
 * This software is the property of Advanced Micro Devices, Inc  (AMD)  which
 * specifically  grants the user the right to modify, use and distribute this
 * software provided this notice is not removed or altered.  All other rights
 * are reserved by AMD.
 *
 * AMD MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH REGARD TO THIS
 * SOFTWARE.  IN NO EVENT SHALL AMD BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES IN CONNECTION WITH OR ARISING FROM THE FURNISHING, PERFORMANCE, OR
 * USE OF THIS SOFTWARE.
 *
 * Comments about this software should be directed to udi@amd.com. If access
 * to electronic mail isn't available, send mail to:
 *
 * Advanced Micro Devices, Inc.
 * 29K Support Products
 * Mail Stop 573
 * 5900 E. Ben White Blvd.
 * Austin, TX 78741
 *****************************************************************************
 *       $Id: udip2dos.c,v 1.2.1.7 1992/02/05 02:53:38 richard Exp $
 *       $Id: @(#)udip2dos.c	2.11, AMD
 */
#if 1
#define DFEIPCIdCompany 0x0001	/* Company ID AMD */
#define DFEIPCIdProduct 0x1	/* Product ID 0 */
#endif
#define DFEIPCIdVersion 0x125	/* 1.2.5 */

#include <process.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <udiproc.h>
#include <udidos.h>

#define BUFSIZE 2048

struct UDIConnections {
    struct UDIVecRec far * VecRecP;
    UDISessionId ConnId;
    };

#define NumSessions 10

static struct UDIVecRec far * CurrentConnection;
static struct UDIVecRec far * far * UDIVecP;
static struct UDIConnections Connections[ NumSessions ];

static char *GetConfigFileName( void )
{
    char *cp;
    static char buf[ _MAX_PATH ];

    if ((cp = getenv( "UDICONF" )) != NULL)
	return cp;
    
    _searchenv( "udiconfs.txt", "PATH", buf );

    return buf;
    }

static struct UDIVecRec far * far * FindIntVect()
{
    union rec recognizer;

    InitRecognizer( &recognizer );

    /* Try and find a vector that is currently in use for UDI */
    for (UDIVecP =  (struct UDIVecRec far * far *)(0x60*4);
	 UDIVecP <= (struct UDIVecRec far * far *)(0x66*4);
	 UDIVecP++) {
	if ((*UDIVecP != NULL) && (*UDIVecP)->recognizer.l == recognizer.l)
	    return UDIVecP;
	}

    return NULL;
    }

extern struct DOSTerm TermStruct;

UDIError UDIConnect UDIParams((
  char		*Configuration,		/* In */
  UDISessionId	*Session		/* Out */
  ))
{
    int i, n;
    char buf[ BUFSIZE ];
    char TIPName[ FILENAME_MAX ];
    char *TIPParms;
    struct UDIVecRec far * CurConn;
    FILE *fp;

    n = strlen( Configuration );
    TIPParms = NULL;
    CurConn = NULL;

    /* Get configuration line that corresponds to requested configuration */
    if ((fp = fopen( GetConfigFileName(), "r" )) == NULL)
	return UDIErrorCantOpenConfigFile;

    while (fgets( buf, BUFSIZE -1, fp ) != NULL) {
	if (isspace( buf[n] ) && (strncmp( buf, Configuration, n ) == 0)) {
	    sscanf( buf, "%*s %s%n", TIPName, &n );
	    TIPParms = &buf[n];
	    while (isspace( *TIPParms ))
		TIPParms++;
	    if (*TIPParms)
		TIPParms[ strlen( TIPParms ) -1 ] = '\0';
	    break;
	    }
	}

    fclose( fp );

    if (!TIPParms)
	return UDIErrorNoSuchConfiguration;

    for (i = 0; i < NumSessions; i++)
	if (!Connections[i].VecRecP)
	    break;
    
    if (i >= NumSessions)
	return UDIErrorCantConnect;

    /* Determine if any TIPs are currently running */
    if (!UDIVecP)	/* None for this DFE */
	UDIVecP = FindIntVect();	/* Try other DFEs */

    if (UDIVecP)	/* TIPs running somewhere on machine */
	for (CurConn = *UDIVecP; CurConn; CurConn = CurConn->Next)
	    if (!_fstrcmp( TIPName, CurConn->exeName )) {
		n = CurConn->UDIConnect( TIPParms, &Connections[i].ConnId,
						&TermStruct );
		if (n == UDINoError) {
		    *Session = i;
		    CurrentConnection = Connections[i].VecRecP = CurConn;
		    return n;
		    }
		if (n != UDIErrorTryAnotherTIP)
		    return n;
		}

    /* To get around DOS 2.x problems as well as problems with filename
       expansion/searching, we pass the executable name as an addtional
       parameter. Also, we expect the TIP to TSR, so the high byte of
       the return value is probably 0x03. But some TIPs might be more
       clever and really return (activating a device driver or other
       TSR first), so we ignore the high byte of the return from spawnlp.
    */
    if ((n = spawnlp( P_WAIT, TIPName, TIPName, TIPName, NULL ) & 0xff) != 0)
	return UDIErrorCantStartTIP;

    /* Determine if any TIPs are currently running */
    if (!UDIVecP)	/* None for this DFE */
	UDIVecP = FindIntVect();	/* Try other DFEs (or new TIPs) */

    if (UDIVecP)	/* TIPs running somewhere on machine */
	for (CurConn = *UDIVecP; CurConn; CurConn = CurConn->Next)
	    if (!_fstrcmp( TIPName, CurConn->exeName )) {
		n = CurConn->UDIConnect( TIPParms, &Connections[i].ConnId,
						&TermStruct );
		if (n <= UDINoError) {
		     /* Note: negative errors need to allow DFE to call GetErrorMsg */
		    *Session = i;
		    CurrentConnection = Connections[i].VecRecP = CurConn;
		    }
		if (n != UDIErrorTryAnotherTIP)
		    return n;
		}

    return UDIErrorExecutableNotTIP;
    }

UDIError UDIDisconnect UDIParams((
  UDISessionId	Session,		/* In */
  UDIBool	Terminate		/* In */
  ))
{
    UDIError err;

    if (Session >= NumSessions || !Connections[Session].VecRecP)
	return UDIErrorNoSuchConnection;

    if (((err = CurrentConnection->UDIDisconnect(
		Connections[ Session ].ConnId, Terminate, &TermStruct ))
		== UDINoError) &&
		(CurrentConnection == Connections[ Session ].VecRecP))
	CurrentConnection = NULL;

    return err;
    }

UDIError UDISetCurrentConnection UDIParams((
  UDISessionId	Session			/* In */
  ))
{
    UDIError n;
    struct UDIConnections *ConnP;

    if (Session >= NumSessions || !Connections[Session].VecRecP)
	return UDIErrorNoSuchConnection;

    ConnP = &Connections[Session];
    if ( (n = ConnP->VecRecP->UDISetCurrentConnection( ConnP->ConnId ))
	    == UDINoError)
	CurrentConnection = ConnP->VecRecP;

    return n;
    }

UDIError UDICapabilities UDIParams((
  UDIUInt32	*TIPId,			/* Out */
  UDIUInt32	*TargetId,		/* Out */
  UDIUInt32	DFEId,			/* In */
  UDIUInt32	DFE,			/* In */
  UDIUInt32	*TIP,			/* Out */
  UDIUInt32	*DFEIPCId,		/* Out */
  UDIUInt32	*TIPIPCId,		/* Out */
  char		*TIPString		/* Out */
  ))
{
    UDIError err;

    if (!CurrentConnection)
	return UDIErrorNoConnection;

    err = CurrentConnection->UDICapabilities( TIPId, TargetId, DFEId, DFE,
	    TIP, DFEIPCId, TIPIPCId, TIPString );

    *DFEIPCId = (((UDIUInt32)DFEIPCIdCompany) << 16) |
			(DFEIPCIdProduct << 12) | DFEIPCIdVersion;

    return err;
    }

UDIError UDIEnumerateTIPs UDIParams((
  UDIInt	(*UDIETCallback)	/* In */
    UDIParams(( char *Configuration ))	/* In to callback() */
  ))
{
    FILE *fp;
    char buf[ BUFSIZE ];

    if ((fp = fopen( GetConfigFileName(), "r" )) == NULL)
	return UDIErrorCantOpenConfigFile;

    while (fgets( buf, BUFSIZE, fp ))
	if (UDIETCallback( buf ) == UDITerminateEnumeration)
	    break;

    fclose( fp );

    return UDINoError;
    }

UDIError UDIGetErrorMsg UDIParams((
  UDIError	ErrorCode,		/* In */
  UDISizeT	MsgSize,		/* In */
  char		*Msg,			/* Out */
  UDISizeT	*CountDone		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIGetErrorMsg( ErrorCode, MsgSize,
	    Msg, CountDone );
    }

UDIError UDIGetTargetConfig UDIParams((
  UDIMemoryRange KnownMemory[],		/* Out */
  UDIInt	*NumberOfRanges,	/* In/Out */
  UDIUInt32	ChipVersions[],		/* Out */
  UDIInt	*NumberOfChips		/* In/Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIGetTargetConfig( KnownMemory,
		    NumberOfRanges, ChipVersions, NumberOfChips );
    }

UDIError UDICreateProcess UDIParams((
  UDIPId	*PId			/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDICreateProcess( PId );
    }

UDIError UDISetCurrentProcess UDIParams((
  UDIPId	PId			/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDISetCurrentProcess( PId );
    }

UDIError UDIDestroyProcess UDIParams((
  UDIPId	PId			/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIDestroyProcess( PId );
    }

UDIError UDIInitializeProcess UDIParams((
  UDIMemoryRange ProcessMemory[],	/* In */
  UDIInt	NumberOfRanges,		/* In */
  UDIResource	EntryPoint,		/* In */
  CPUSizeT	StackSizes[],		/* In */
  UDIInt	NumberOfStacks,		/* In */
  char		*ArgString		/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIInitializeProcess( ProcessMemory,
	NumberOfRanges, EntryPoint, StackSizes, NumberOfStacks, ArgString );
    }

UDIError UDIRead UDIParams((
  UDIResource	From,			/* In */
  UDIHostMemPtr	To,			/* Out */
  UDICount	Count,			/* In */
  UDISizeT	Size,			/* In */
  UDICount	*CountDone,		/* Out */
  UDIBool	HostEndian		/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIRead( From, To, Count, Size,
	    CountDone, HostEndian );
    }

UDIError UDIWrite UDIParams((
  UDIHostMemPtr	From,			/* In */
  UDIResource	To,			/* In */
  UDICount	Count,			/* In */
  UDISizeT	Size,			/* In */
  UDICount	*CountDone,		/* Out */
  UDIBool	HostEndian		/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIWrite( From, To, Count, Size,
	    CountDone, HostEndian );
    }

UDIError UDICopy UDIParams((
  UDIResource	From,			/* In */
  UDIResource	To,			/* In */
  UDICount	Count,			/* In */
  UDISizeT	Size,			/* In */
  UDICount	*CountDone,		/* Out */
  UDIBool	Direction		/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDICopy( From, To, Count, Size,
	    CountDone, Direction );
    }

UDIError UDIExecute UDIParams((
  void
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIExecute();
    }

UDIError UDIStep UDIParams((
  UDIUInt32	Steps,			/* In */
  UDIStepType   StepType,		/* In */
  UDIRange      Range			/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIStep( Steps, StepType, Range );
    }

UDIVoid UDIStop UDIParams((
  void
  ))
{
    if (!CurrentConnection)
	return;

    CurrentConnection->UDIStop();
}

UDIError UDIWait UDIParams((
  UDIInt32	MaxTime,		/* In */
  UDIPId	*PId,			/* Out */
  UDIUInt32	*StopReason		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIWait( MaxTime, PId, StopReason );
    }

UDIError UDISetBreakpoint UDIParams((
  UDIResource	Addr,			/* In */
  UDIInt32	PassCount,		/* In */
  UDIBreakType	Type,			/* In */
  UDIBreakId	*BreakId		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDISetBreakpoint( Addr, PassCount,
	    Type, BreakId );
    }

UDIError UDIQueryBreakpoint UDIParams((
  UDIBreakId	BreakId,		/* In */
  UDIResource	*Addr,			/* Out */
  UDIInt32	*PassCount,		/* Out */
  UDIBreakType	*Type,			/* Out */
  UDIInt32	*CurrentCount		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIQueryBreakpoint( BreakId, Addr,
	    PassCount, Type, CurrentCount );
    }

UDIError UDIClearBreakpoint UDIParams((
  UDIBreakId	BreakId			/* In */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIClearBreakpoint( BreakId );
    }

UDIError UDIGetStdout UDIParams((
  UDIHostMemPtr	Buf,			/* Out */
  UDISizeT	BufSize,		/* In */
  UDISizeT	*CountDone		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIGetStdout( Buf, BufSize, CountDone );
    }

UDIError UDIGetStderr UDIParams((
  UDIHostMemPtr	Buf,			/* Out */
  UDISizeT	BufSize,		/* In */
  UDISizeT	*CountDone		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIGetStderr( Buf, BufSize, CountDone );
    }

UDIError UDIPutStdin UDIParams((
  UDIHostMemPtr	Buf,			/* In */
  UDISizeT	Count,			/* In */
  UDISizeT	*CountDone		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIPutStdin( Buf, Count, CountDone );
    }

UDIError UDIStdinMode UDIParams((
  UDIMode	*Mode			/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIStdinMode( Mode );
    }

UDIError UDIPutTrans UDIParams((
  UDIHostMemPtr	Buf,			/* In */
  UDISizeT	Count,			/* In */
  UDISizeT	*CountDone		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIPutTrans( Buf, Count, CountDone );
    }

UDIError UDIGetTrans UDIParams((
  UDIHostMemPtr	Buf,			/* Out */
  UDISizeT	BufSize,		/* In */
  UDISizeT	*CountDone		/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDIGetTrans( Buf, BufSize, CountDone );
    }

UDIError UDITransMode UDIParams((
  UDIMode	*Mode			/* Out */
  ))
{
    if (!CurrentConnection)
	return UDIErrorNoConnection;

    return CurrentConnection->UDITransMode( Mode );
    }


UDIUInt32 UDIGetDFEIPCId()
{
    return((((UDIUInt32)DFEIPCIdCompany) << 16) |(DFEIPCIdProduct << 12) | DFEIPCIdVersion);
}
